#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_pymrmr
----------------------------------

Tests for `pymrmr` module.
"""


import sys
import unittest

from pymrmr import mRMR


# TODO

class TestPymrmr(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_000_something(self):
        pass
